# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: triple_index.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='triple_index.proto',
  package='',
  syntax='proto3',
  serialized_pb=_b('\n\x12triple_index.proto\"\xce\x01\n\x0bTranslation\x12,\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x1a.Translation.EntitiesEntry\x12.\n\trelations\x18\x02 \x03(\x0b\x32\x1b.Translation.RelationsEntry\x1a/\n\rEntitiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\x1a\x30\n\x0eRelationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\x62\x06proto3')
)




_TRANSLATION_ENTITIESENTRY = _descriptor.Descriptor(
  name='EntitiesEntry',
  full_name='Translation.EntitiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Translation.EntitiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='Translation.EntitiesEntry.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=132,
  serialized_end=179,
)

_TRANSLATION_RELATIONSENTRY = _descriptor.Descriptor(
  name='RelationsEntry',
  full_name='Translation.RelationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Translation.RelationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='Translation.RelationsEntry.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=181,
  serialized_end=229,
)

_TRANSLATION = _descriptor.Descriptor(
  name='Translation',
  full_name='Translation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entities', full_name='Translation.entities', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relations', full_name='Translation.relations', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TRANSLATION_ENTITIESENTRY, _TRANSLATION_RELATIONSENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23,
  serialized_end=229,
)

_TRANSLATION_ENTITIESENTRY.containing_type = _TRANSLATION
_TRANSLATION_RELATIONSENTRY.containing_type = _TRANSLATION
_TRANSLATION.fields_by_name['entities'].message_type = _TRANSLATION_ENTITIESENTRY
_TRANSLATION.fields_by_name['relations'].message_type = _TRANSLATION_RELATIONSENTRY
DESCRIPTOR.message_types_by_name['Translation'] = _TRANSLATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Translation = _reflection.GeneratedProtocolMessageType('Translation', (_message.Message,), dict(

  EntitiesEntry = _reflection.GeneratedProtocolMessageType('EntitiesEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRANSLATION_ENTITIESENTRY,
    __module__ = 'triple_index_pb2'
    # @@protoc_insertion_point(class_scope:Translation.EntitiesEntry)
    ))
  ,

  RelationsEntry = _reflection.GeneratedProtocolMessageType('RelationsEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRANSLATION_RELATIONSENTRY,
    __module__ = 'triple_index_pb2'
    # @@protoc_insertion_point(class_scope:Translation.RelationsEntry)
    ))
  ,
  DESCRIPTOR = _TRANSLATION,
  __module__ = 'triple_index_pb2'
  # @@protoc_insertion_point(class_scope:Translation)
  ))
_sym_db.RegisterMessage(Translation)
_sym_db.RegisterMessage(Translation.EntitiesEntry)
_sym_db.RegisterMessage(Translation.RelationsEntry)


_TRANSLATION_ENTITIESENTRY.has_options = True
_TRANSLATION_ENTITIESENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_TRANSLATION_RELATIONSENTRY.has_options = True
_TRANSLATION_RELATIONSENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
